/***************************************************************************
                          game.cpp  -  description
                             -------------------
    begin                : Tue Aug 28 2001
    copyright            : (C) 2001 by Giuseppe D'Aqu
    email                : kumber@tiscalinet.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "dephine.h"
#include "sprite.h"
#include <vector>
#include <cmath>
#include <fstream>
#include "entity_type.h"
#include "entity.h"
#include "entity_player.h"
#include "game.h"
#include "level.h"
#include <stdlib.h>

bool Game::main_loop()
{
	unsigned int current_frame_time=0;
	
	bool exit_state=false;
	
	m_screen.set_window_center(m_level->get_player().get_position_x()*k_sprite_size,m_level->get_player().get_position_y()*k_sprite_size);
	

	while((m_level->get_player().is_alive())||
				(!CL_Keyboard::get_keycode(CL_KEY_SPACE))
				
				)
	{
		current_frame_time=CL_System::get_time();
		
	//	m_level->get_player().set_direction(STOP);
		
		get_keys();
		
		move_all();
		
		draw();
		
		CL_System::keep_alive();

	}
	
	exit_state=m_level->get_player().is_exited();	
	
	return exit_state;
	
}

void Game::get_keys()
{
	//direzione iniziale
	CL_System::keep_alive();
	
	Direction direction/*=STOP*/;
	
	if((CL_Keyboard::get_keycode(CL_KEY_ESCAPE))&&(m_level->get_player().is_alive()))
	{
			
		m_level->explode(m_level->get_player().get_position_x(),m_level->get_player().get_position_y());
		return;
				
	}
	
  //questi if settano la direzione a seconda del tasto premuto
	if(CL_Keyboard::get_keycode(CL_KEY_UP))
	{
			
		direction=UP;
				
	}
	else if(CL_Keyboard::get_keycode(CL_KEY_DOWN))
	{
			
		direction=DOWN;
		
	}
	else if(CL_Keyboard::get_keycode(CL_KEY_RIGHT))
	{
	
		direction=RIGHT;
		
	}
	else if(CL_Keyboard::get_keycode(CL_KEY_LEFT))
	{
	
		direction=LEFT;
		
	}
	if((direction>=UP)&&(direction<=LEFT))
	{
		m_level->get_player().set_direction(direction);
		DEBOUT("Setting direction: "<<direction<<"\n");
	}
	
	if(CL_Keyboard::get_keycode(CL_KEY_RCTRL))
	{
		m_level->get_player().set_snap(true);
	}
	else
	{
		m_level->get_player().set_snap(false);
	}
	
	
}

void Game::move_all()
{
	
	std::vector<Ntt_pointer>& list=m_level->get_entities_list();
	std::vector< std::vector<int> >& matrix=m_level->get_entities_matrix();
	unsigned int i,x,y;
	for(x=0;x<k_level_size_x;x++)
	{
		for(y=0;y<k_level_size_y;y++)
		{
			matrix[x][y]=0;
		}
	}
	
	for(i=1; i<list.size(); i++)
	{	
		if((list[i].is_referenced())&&(list[i]->exists()))
		{
			list[i]->set_checked(false);
			matrix[list[i]->get_position_x()][list[i]->get_position_y()]=list[i]->get_id();
		}
	}
	
	
	for(i=1; i<list.size(); i++)
	{
		if((list[i].is_referenced())&&(list[i]->exists()))
		{
			if(list[i]->get_type()!=PLAYER)
				list[i]->check_and_do();
		}
	}
	
	if(m_level->get_player().exists())
		m_level->get_player().check_and_do();
	
	
}

void Game::draw()
{
//	Sprite* curr_sprite;
	std::vector<Ntt_pointer>& list=m_level->get_entities_list();
	Ntt_pointer curr_ntt;
	unsigned int current_frame_time=0;
	for(unsigned int j=0;j<k_max_anim_drawn;j++)
	{
		current_frame_time=CL_System::get_time();
		CL_Display::clear_display(0.0, 0.0, 0.0, 1.0);
		m_screen.set_window_center(m_level->get_player().get_sprite().get_pos_x(),m_level->get_player().get_sprite().get_pos_y());
		if(m_level->get_player().exists())
		{
			m_level->get_player().get_sprite().set_curr_frame(j);
			m_screen.put(m_level->get_player().get_sprite());

		}
		for(unsigned int i=1; i<list.size(); i++)
		{
			curr_ntt=list[i];
			if((curr_ntt.is_referenced())&&(curr_ntt->exists())&&(curr_ntt->get_type()!=PLAYER))
			{
				curr_ntt->get_sprite().set_curr_frame(j);
				
				m_screen.put(curr_ntt->get_sprite());
			}
		}
		
		draw_score();									
		CL_Display::flip_display(true);
		if(j==k_max_anim/2)
		{
			
			get_keys();
		}
				
		while(CL_System::get_time()-current_frame_time<k_msec_per_frame);

	}
		
	CL_System::keep_alive();
	
}

void Game::draw_score()
{

	CL_Display::fill_rect(0, k_game_size_y-k_score_size_y, k_game_size_x, k_game_size_y,0,0,0,1.0);

		
	m_game_font->print_left(4,k_game_size_y-k_score_size_y+5, CL_String("Score:   ")+CL_String((int)m_level->get_player().get_score()));
	
	m_game_font->print_left(200,k_game_size_y-k_score_size_y+5, CL_String("Remaining:   ")+CL_String((int)(m_level->get_min_score()-m_level->get_player().get_score())));
	
	if(!m_level->get_player().is_alive())
		m_game_font->print_left(380,k_game_size_y-k_score_size_y+5, CL_String("Press Space"));
	
	
	if(m_level->get_acquired_keys()&1)
		m_sprites[KEY]->put_screen(k_game_size_x-k_sprite_size,k_game_size_y-k_sprite_size, k_sprite_size,k_sprite_size);
	
	CL_Display::draw_rect(0, k_game_size_y-k_score_size_y, k_game_size_x, k_game_size_y,1,0.5,0,1.0);

}

void Game::go()
{

//	DEBOUT("Entering Game::go()...\n");
	
//	DEBOUT("Entering main loop...\n");
	
	int play=menu();
	
	while(play!=3)
	{
		if(play==0)
		{
			m_level=new Level(m_sprites);
			
			CL_String current_level_path(m_resource_path);
			
			current_level_path=current_level_path+"/maps/level";
			
			current_level_path=current_level_path+((const int)m_current_level);
			
			current_level_path+=".map";
		
			try
			{
				m_level->load_map(current_level_path);
									
			}
			catch(Common_Ex e)
			{
				std::cout<<e.get_message();
				return;
			}
		
			bool result=main_loop();
		
			CL_System::sleep(500);
			
			CL_System::keep_alive();	
			//HACKSOMETHINGHERE
			//Might add some Hiscores here...
			delete m_level;
			
			switch(result)
			{
			case false:
				DEBOUT("Game_over!...");
				play=menu();
				break;
			case true:
				DEBOUT("Winner! ;)");
					
				if(m_current_level<k_num_max_of_levels-1)
				{
					play=0;
					if(m_current_level==m_unsolved_level)
					{
						m_unsolved_level++;
						ofstream update_config(m_ini_path);
						update_config<<m_unsolved_level;
					}
					m_current_level++;
				}
				else
				{
					play=menu();
				}
			
				break;
			}		
		}
		else if(play==2)
		{
			show_credits();
			play=menu();
		}	
	}
}

	

Game::Game()
{

	DEBOUT("Entering Game ctor...\n");
	//portability "quick hack" :)
	#ifdef _WIN32
	m_resource_path=".";
	#else
	m_resource_path= RESOURCE_PATH;
	#endif
	DEBOUT("Resource_path: "<<m_resource_path<<"\n");

	m_current_level=0;
	
	
	
	#ifdef _WIN32
	m_ini_path="./epiphany.ini";
	#else
	CL_String user_home=getenv("HOME");
	if(user_home=="")
	{
		DEBOUT("Unable to find HOME environment variable");
	}
	else
	{
		m_ini_path=user_home+"/.epiphany";
	}
	#endif
	
	load_config();	
		
	m_current_level=m_unsolved_level;

	load_sprites();
		
	m_screen.init(k_game_size_x,k_game_size_y,k_level_size_x, k_level_size_y, k_sprite_size);
		
	DEBOUT("Exiting Game ctor... \n");
	
}

void Game::load_config()
{
	std::ifstream config(m_ini_path);
	
	if(!config)
	{
		DEBOUT("No "<<m_ini_path<<"; Creating...\n");
		std::ofstream new_config(m_ini_path);
		if(!new_config)
			DEBOUT("Unable to write!...\n");
		m_unsolved_level=0;
		new_config<<m_unsolved_level;
		return;
	}
	config>>m_unsolved_level;
}
	

void Game::load_sprites()
{

	CL_ResourceManager manag(m_resource_path+"/sprites.scr", false);

	m_menu_font=CL_Font::load("Fonts/FNT_Menu", &manag);
	
	m_game_font=CL_Font::load("Fonts/FNT_Game", &manag);
		
	m_sprites.push_back(Sprite(new CL_Surface("Surfaces/SPT_Unknown", &manag)));
	
	m_sprites.push_back(Sprite(new CL_Surface("Surfaces/SPT_Player", &manag)));

	m_sprites.push_back(Sprite(new CL_Surface("Surfaces/SPT_Grass", &manag)));
	
	m_sprites.push_back(Sprite(new CL_Surface("Surfaces/SPT_Steel", &manag)));
	
	m_sprites.push_back(Sprite(new CL_Surface("Surfaces/SPT_Emerald", &manag)));
	
	m_sprites.push_back(Sprite(new CL_Surface("Surfaces/SPT_Boulder", &manag)));

	m_sprites.push_back(Sprite(new CL_Surface("Surfaces/SPT_Sapphire", &manag)));
		
	m_sprites.push_back(Sprite(new CL_Surface("Surfaces/SPT_Exit", &manag)));
	
	m_sprites.push_back(Sprite(new CL_Surface("Surfaces/SPT_Explosion", &manag)));
	
	m_sprites.push_back(Sprite(new CL_Surface("Surfaces/SPT_Door", &manag)));

	m_sprites.push_back(Sprite(new CL_Surface("Surfaces/SPT_Key", &manag)));	

	m_sprites.push_back(Sprite(new CL_Surface("Surfaces/SPT_Flintstone", &manag)));	
	
	m_sprites.push_back(Sprite(new CL_Surface("Surfaces/SPT_Peperon", &manag)));	
	
	m_sprites.push_back(Sprite(new CL_Surface("Surfaces/SPT_Brick", &manag)));
	
	m_sprites.push_back(Sprite(new CL_Surface("Surfaces/SPT_Wood", &manag)));
	
	
}

Game::~Game()
{
	delete m_menu_font;
	delete m_game_font;
	
}

int Game::menu()
{
	DEBOUT("Entering menu()...\n");
	int selected=0;
	unsigned int level_number=m_current_level;
	unsigned int current_time;
	CL_String level_string;
	
	CL_System::keep_alive();
	while(!(((CL_Keyboard::get_keycode(CL_KEY_ENTER))||
				(CL_Keyboard::get_keycode(CL_KEY_SPACE)))&&
				(selected!=1)) )

	{
		current_time=CL_System::get_time();
		CL_Display::clear_display(0.0, 0.0, 0.0, 1.0);
		level_string=(int)level_number;
		m_menu_font->print_center(k_game_size_x/2,30,"Epiphany");
		m_menu_font->print_left((k_game_size_x/2)-50, 200, "Start");
		m_menu_font->print_left((k_game_size_x/2)-50, 260, "Level:");
		m_menu_font->print_left((k_game_size_x/2)+60, 260, level_string);
		m_menu_font->print_left((k_game_size_x/2)-50, 320, "Credits");
		m_menu_font->print_left((k_game_size_x/2)-50, 380, "Quit");
		if(CL_Keyboard::get_keycode(CL_KEY_UP))
		{
			if(selected>0)
			{
				selected--;
			}
		}
		
		if(CL_Keyboard::get_keycode(CL_KEY_DOWN))
		{
			if(selected<3)
			{
				selected++;
			}
		}
		
		if((CL_Keyboard::get_keycode(CL_KEY_LEFT))&&
				(selected==1)&&
				(level_number>0))
		{
			level_number--;
		}
		
		if((CL_Keyboard::get_keycode(CL_KEY_RIGHT))&&
				(selected==1)&&
				(level_number<k_num_max_of_levels-1)&&
				(level_number<m_unsolved_level))
		{
			level_number++;
		}
		
		
		m_sprites[EMERALD]->put_screen((k_game_size_x/2)-90,200+selected*60,32,32,0);  			
		
		CL_Display::flip_display();
		do
		{
			CL_System::keep_alive();
		}
		while((CL_System::get_time()-current_time)<100);
	}	
	m_current_level=level_number;
	//if(selected==0)
	//	return true;
	//return false;
	DEBOUT("Exiting menu()... "<<selected<<"\n");
	return selected;
}

void Game::show_credits()
{
	int current_frame_time=0;
	vector<CL_String> credits;
	credits.push_back("Epiphany");
	credits.push_back("A game by Giuseppe D'Aqu");
	credits.push_back(" ");
	credits.push_back("Coding:");
	credits.push_back("Giuseppe D'Aqu");
	credits.push_back(" ");
	credits.push_back("Level Designing:");
	credits.push_back("Giuseppe D'Aqu");
	credits.push_back("Giuseppe Martino");
	credits.push_back(" ");
	credits.push_back("Object Oriented Light:");
	credits.push_back("Giuseppe Martino");
	credits.push_back(" ");
	credits.push_back("Graphics:");
	credits.push_back("Giuseppe D'Aqu");
	credits.push_back(" ");
	credits.push_back(" ");
	credits.push_back("Thanks to:");
//	credits.push_back("Antonio \"Willy\" Malara");
	credits.push_back("Kenneth Gangstoe");
	credits.push_back("Joerg Jaspert");
	credits.push_back("Magnus Norddahl");
	credits.push_back("and all ClanLib developers");
	
	int draw_pos=k_game_size_y;
	unsigned int i;
	while(!CL_Keyboard::get_keycode(CL_KEY_ESCAPE))
	{
		current_frame_time=CL_System::get_time();
		CL_Display::clear_display(0,0,0,1);
		for(i=0; i<credits.size(); i++)
		{
		//	m_menu_font->print_center(k_game_size_x/2,draw_pos+50*i,credits[i]);
			m_menu_font->print_center((k_game_size_x/2)-20*sin(M_PI*20*draw_pos/k_game_size_y+(i%4)*M_PI/4),draw_pos+50*i,credits[i]);
		
		}
		if(draw_pos+50*credits.size()+100>0)
		{
			draw_pos--;
		}
		else
		{
			draw_pos=k_game_size_y+100;
		}
		CL_Display::flip_display(true);
		while(CL_System::get_time()-current_frame_time<23);

		CL_System::keep_alive();
	}
		
}






