/***************************************************************************
                          level.cpp  -  description
                             -------------------
    begin                : Fri Aug 31 2001
    copyright            : (C) 2001 by Giuseppe D'Aqu
    email                : kumber@tiscalinet.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dephine.h"
#include "entity.h"
#include "entity_all.h"
#include "entity_player.h"
#include "level.h"
#include "screen.h"
#include "sprite.h"
#include <vector>
//#include <map>
#include <fstream>

extern int glb_curr_id;

Level::Level(std::vector<Sprite>& sprites):m_sprites(sprites)
{
	
	
	m_entities_list.push_back(Ntt_pointer());
	m_entities_matrix.resize(k_level_size_x);
	for(int i=0; i<k_level_size_x; i++)
	{
		m_entities_matrix[i].resize(k_level_size_y);
	}
	glb_curr_id=1;
	m_acquired_keys=0;
	m_min_score=0;
	
}

void Level::set_key(unsigned int key)
{
	m_acquired_keys=m_acquired_keys|key;
}

int Level::get_entity_id(int x, int y)
{
	//HCKSMTHNGHR
	//check limits
	if((x<0)||(y<0))
	{
		DEBOUT("Something outside limits...\n");
		return 0;
	}
	return m_entities_matrix[x][y];
}

int Level::get_entity_id(int x, int y, Direction d)
{
	int dx, dy;
	
	switch(d)
	{
	case STOP:
		dx=0;
		dy=0;
		break;
	case UP:
		dx=0;
		dy=-1;
		break;
	case UPRIGHT:
		dx=1;
		dy=-1;
		break;	
	case RIGHT:
		dx=1;
		dy=0;
		break;
	case DOWNRIGHT:
		dx=1;
		dy=1;
		break;	
	case DOWN:
		dx=0;
		dy=1;
		break;
	case DOWNLEFT:
		dx=-1;
		dy=1;
		break;	
	case LEFT:
		dx=-1;
		dy=0;
		break;
	case UPLEFT:
		dx=-1;
		dy=-1;
		break;	
	
	}
	
	return get_entity_id(x+dx, y+dy);
}

Ntt_pointer& Level::get_entity(int ntt_id)
{
	return m_entities_list[ntt_id];
}

Ntt_pointer& Level::get_entity(int x, int y)
{
	return m_entities_list[m_entities_matrix[x][y]];
}

Ntt_pointer& Level::get_entity(int x, int y, Direction d)
{
	int dx, dy;
	
	switch(d)
	{
	case STOP:
		dx=0;
		dy=0;
		break;
	case UP:
		dx=0;
		dy=-1;
		break;
	case UPRIGHT:
		dx=1;
		dy=-1;
		break;	
	case RIGHT:
		dx=1;
		dy=0;
		break;
	case DOWNRIGHT:
		dx=1;
		dy=1;
		break;	
	case DOWN:
		dx=0;
		dy=1;
		break;
	case DOWNLEFT:
		dx=-1;
		dy=1;
		break;	
	case LEFT:
		dx=-1;
		dy=0;
		break;
	case UPLEFT:
		dx=-1;
		dy=-1;
		break;	
	
	}
	
	return get_entity(x+dx, y+dy);

}


std::vector<Ntt_pointer>& Level::get_entities_list()
{
	return m_entities_list;
}

void Level::load_map(const char* map_path)
{
	Entity_Player* pl;	
	
	Entity_Exit* ex;
	
	ifstream map(map_path);
	
	if(!map)
		Common_Ex::throw_common(CL_String("Error loading map in ")+map_path+CL_String("\n"));
	
	DEBOUT("Loading map in "<<map_path<<" ...\n");
	
//	unsigned int min_score=0;
	
	char cell;
	/*HCKSMTHNGHR*/		
	for(int j=0; j<k_level_size_y; j++)
		for(int i=0; i<k_level_size_x; i++)
		{
			map.get(cell);
	//		DEBOUT(cell<<"\n");
			if(cell==0x0a)
			{
				map.get(cell);
			}
			
			switch(cell)
			{
			case '0':
				//do nothing
				break;
			case '1':
				pl=new Entity_Player(this,i,j,m_sprites[PLAYER]);
				set_player(pl);
				break;
			case '2':
				set_entity(new Entity_Grass(this,i,j,m_sprites[GRASS]));
				break;
			case '3':
				set_entity(new Entity_Steel(this,i,j,m_sprites[STEEL]));
				break;
			case '4':
				set_entity(new Entity_Emerald(this,i,j,m_sprites[EMERALD]));
				m_min_score++;
				break;
			case '5':
				set_entity(new Entity_Boulder(this,i,j,m_sprites[BOULDER]));
				break;
			case '6':
				set_entity(new Entity_Sapphire(this,i,j,m_sprites[SAPPHIRE]));
				m_min_score+=3;
				break;
			case '7':
				ex=new Entity_Exit(this,i,j,m_sprites[EXIT],1);
				set_entity(ex);
				break;
			case '9':
				set_entity(new Entity_Door(this,i,j,m_sprites[DOOR],1));
				break;
			case 'a':
				set_entity(new Entity_Key(this,i,j,m_sprites[KEY],1));
				break;
			case 'b':
				set_entity(new Entity_Flintstone(this,i,j,m_sprites[FLINTSTONE]));
				break;
			case 'c':
				set_entity(new Entity_Peperon(this,i,j,m_sprites[PEPERON]));
				break;
			case 'd':
				set_entity(new Entity_Brick(this,i,j,m_sprites[BRICK]));
				break;
			case 'e':
				set_entity(new Entity_Wood(this,i,j,m_sprites[WOOD]));
				break;
			default:
				//do nothing
				break;
			}
		}
	
	ex->set_min_score(m_min_score);
	
}	

void Level::set_player(Entity_Player* pl)
{
	m_player=pl;
//	m_entities_list[m_player->get_id()]=Ntt_pointer(*m_player);
	m_entities_list.push_back(Ntt_pointer(*m_player));
	m_entities_matrix[pl->get_position_x()][pl->get_position_y()]=m_player->get_id();
}

void Level::set_entity(Entity* ntt)
{
//	m_entities_list[ntt->get_id()]=Ntt_pointer(*ntt);
	m_entities_list.push_back(Ntt_pointer(*ntt));
	m_entities_matrix[ntt->get_position_x()][ntt->get_position_y()]=ntt->get_id();
}

int Level::get_size_x()
{
	return k_level_size_x;
}

int Level::get_size_y()
{
	return k_level_size_y;
}

std::vector<std::vector<int> >& Level::get_entities_matrix()
{
	return m_entities_matrix;
}

Entity_Player& Level::get_player()
{
	return *m_player;
}

Sprite& Level::get_sprite(Entity_Type entity_type)
{

	return (m_sprites)[entity_type];
	
}

Level::~Level()
{
//	for (unsigned int i=0; i<m_entities_list.size(); i++)
//		m_entities_list[i].remove();
}
	
unsigned int Level::get_acquired_keys()
{
	return m_acquired_keys;
}

void Level::explode(unsigned int x, unsigned int y)
{
//	DEBOUT("Entering Level::explode("<<x<<", "<<y<<")\n");
//	DEBOUT("id: "<<m_entities_matrix[x][y]<<"\n");
	Ntt_pointer& curr_ntt=m_entities_list[m_entities_matrix[x][y]];
//	DEBOUT("curr_ntt: id="<<curr_ntt->get_id()<<")\n");
	
	if(curr_ntt.is_referenced())
	{
		if(curr_ntt->exists())
		{
			bool result=curr_ntt->explode();
	//		DEBOUT("result of explode="<<result<<"\n");
			if(result)
			{
				set_entity(new Entity_Explosion(this,x,y,m_sprites[EXPLOSION], Ntt_pointer()));
			}
		}
		else
		{
			set_entity(new Entity_Explosion(this,x,y,m_sprites[EXPLOSION], Ntt_pointer()));
		}
	}
	else
	{
			set_entity(new Entity_Explosion(this,x,y,m_sprites[EXPLOSION], Ntt_pointer()));
	}
		
}

unsigned int Level::get_min_score()
{
	return m_min_score;
}
